//
//  GSCustomParameter.h
//  GlyphsCore
//
//  Created by Georg Seifert on 30.11.12.
//  Copyright (c) 2012 schriftgestaltung.de. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <GlyphsCore/GSBase.h>
#import <GlyphsCore/GSContainerProtocol.h>
#if !defined(GLYPHS_LITE) && !defined(LIBCORE)
#import <GlyphsCore/GSIdentifiableParameter.h>
#endif

NS_ASSUME_NONNULL_BEGIN

@class GSInstance;
@protocol GSContainerProtocol;
@class GSFont;

/** Implementation of the Custom Property object.

 It stores a name/value pair
 */
#if !(defined(GLYPHS_LITE) || defined(LIBCORE))
@interface GSCustomParameter : NSObject <GSIdentifiableParameter>
#else
@interface GSCustomParameter : NSObject
#endif

- (instancetype)initWithName:(NSString *)name value:(id)value;

- (instancetype)initWithDict:(NSDictionary *)dict format:(GSFormatVersion)formatVersion;

@property (weak, nonatomic) GSContainer *parent;
@property (nonatomic, strong) NSString *name;
@property (nonatomic, strong) id value;

@property (nonatomic, strong) id valueUI;

@property (nonatomic, assign) BOOL active;

#ifndef GLYPHS_VIEWER
/// Replaces single quotes with tick marks and removes all non-ASCII characters.
- (NSString *)saveValue;

- (BOOL)saveToFile:(FILE *)file format:(GSFormatVersion)formatVersion error:(NSError **)error;
#endif

+ (NSColor *)colorFromValue:(id)value;
/** If the property stores a color, this returns a NSColor object
 
 It converts a comma separated list of strings into a color. e. g. 63,165,53 will be green
 @return a NSColor object
 */
@property (nonatomic, strong) NSColor *colorValue;

- (void)validateValue;

+ (NSArray *)validateNameList:(NSArray *)parameter;

- (id)propertyListValueFormat:(GSFormatVersion)formatVersion;

+ (id)propertyListValueForColor:(NSColor *)color format:(GSFormatVersion)formatVersion;

- (NSString *)displayValue;

- (id)contentValue;

- (NSString *)stringValue;

- (BOOL)boolValue;

- (float)floatValue;

- (NSInteger)integerValue;

- (int)intValue;

#ifndef GLYPHS_VIEWER
- (CGFloat)localizedValueForScript:(NSString *)script;

- (id)valueAsPropertyListValueFormat:(GSFormatVersion)formatVersion quotesIfNeeded:(BOOL)quotes;

+ (nullable id)infoPropertyForKeys:(nonnull NSArray *)keys fromObjects:(nonnull NSArray *)objects;

+ (nullable GSCustomParameter *)customParameterForKeys:(nonnull NSArray *)keys fromObjects:(nonnull NSArray *)objects;

+ (BOOL)convertParameterToProperties:(NSMutableArray *)parameters object:(GSFont *)object error:(NSError **)error;

+ (void)makeIdentifiers:(NSArray *)parameters;
#endif

- (void)setValueFast:(id)value;
@end

NS_ASSUME_NONNULL_END
